@@ -1,6 +1,6 @@
 # Natural Language Toolkit: CONLL Corpus Reader
 #
-# Copyright (C) 2001-2014 NLTK Project
+# Copyright (C) 2001-2017 NLTK Project
 # Author: Steven Bird <stevenbird1@gmail.com>
 #         Edward Loper <edloper@gmail.com>
 # URL: <http://nltk.org/>
@@ -16,6 +16,8 @@ import os
 import codecs
 import textwrap
 
+from six import string_types
+
 from nltk import compat
 from nltk.tree import Tree
 from nltk.util import LazyMap, LazyConcatenation
@@ -72,7 +74,7 @@ class ConllCorpusReader(CorpusReader):
         for columntype in columntypes:
             if columntype not in self.COLUMN_TYPES:
                 raise ValueError('Bad column type %r' % columntype)
-        if isinstance(chunk_types, compat.string_types):
+        if isinstance(chunk_types, string_types):
             chunk_types = [chunk_types]
         self._chunk_types = chunk_types
         self._colmap = dict((c,i) for (i,c) in enumerate(columntypes))
@@ -89,7 +91,7 @@ class ConllCorpusReader(CorpusReader):
 
     def raw(self, fileids=None):
         if fileids is None: fileids = self._fileids
-        elif isinstance(fileids, compat.string_types): fileids = [fileids]
+        elif isinstance(fileids, string_types): fileids = [fileids]
         return concat([self.open(f).read() for f in fileids])
 
     def words(self, fileids=None):
@@ -280,16 +282,16 @@ class ConllCorpusReader(CorpusReader):
             right = right.count(')')*')' # only keep ')'.
             treestr += '%s (%s %s) %s' % (left, pos_tag, word, right)
         try:
-            tree = self._tree_class.parse(treestr)
+            tree = self._tree_class.fromstring(treestr)
         except (ValueError, IndexError):
-            tree = self._tree_class.parse('(%s %s)' %
+            tree = self._tree_class.fromstring('(%s %s)' %
                                           (self._root_label, treestr))
 
         if not pos_in_tree:
             for subtree in tree.subtrees():
                 for i, child in enumerate(subtree):
                     if (isinstance(child, Tree) and len(child)==1 and
-                        isinstance(child[0], compat.string_types)):
+                        isinstance(child[0], string_types)):
                         subtree[i] = (child[0], child.label())
 
         return tree
@@ -492,7 +494,7 @@ class ConllSRLInstanceList(list):
 
     def _tree2conll(self, tree, wordnum, words, pos, synt):
         assert isinstance(tree, Tree)
-        if len(tree) == 1 and isinstance(tree[0], compat.string_types):
+        if len(tree) == 1 and isinstance(tree[0], string_types):
             pos[wordnum] = tree.label()
             assert words[wordnum] == tree[0]
             return wordnum+1
