@@ -1,13 +1,14 @@
 # Natural Language Toolkit: IPI PAN Corpus Reader
 #
-# Copyright (C) 2001-2015 NLTK Project
+# Copyright (C) 2001-2017 NLTK Project
 # Author: Konrad Goluchowski <kodie@mimuw.edu.pl>
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
 
 import functools
 
-from nltk import compat
+from six import string_types
+
 from nltk.corpus.reader.util import StreamBackedCorpusView, concat
 from nltk.corpus.reader.api import CorpusReader
 
@@ -93,11 +94,11 @@ class IPIPANCorpusReader(CorpusReader):
         if channels is None and domains is None and \
                 categories is None:
             return CorpusReader.fileids(self)
-        if isinstance(channels, compat.string_types):
+        if isinstance(channels, string_types):
             channels = [channels]
-        if isinstance(domains, compat.string_types):
+        if isinstance(domains, string_types):
             domains = [domains]
-        if isinstance(categories, compat.string_types):
+        if isinstance(categories, string_types):
             categories = [categories]
         if channels:
             return self._list_morph_files_by('channel', channels)
