@@ -2,7 +2,7 @@
 
 # Natural Language Toolkit: York-Toronto-Helsinki Parsed Corpus of Old English Prose (YCOE)
 #
-# Copyright (C) 2001-2014 NLTK Project
+# Copyright (C) 2001-2015 NLTK Project
 # Author: Selina Dennis <selina@tranzfusion.net>
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
@@ -22,7 +22,8 @@ to the YCOE standard: http://www-users.york.ac.uk/~lang22/YCOE/YcoeHome.htm
 import os
 import re
 
-from nltk import compat
+from six import string_types
+
 from nltk.tokenize import RegexpTokenizer
 from nltk.corpus.reader.bracket_parse import BracketParseCorpusReader
 from nltk.corpus.reader.tagged import TaggedCorpusReader
@@ -63,7 +64,7 @@ class YCOECorpusReader(CorpusReader):
         """
         if fileids is None:
             return self._documents
-        if isinstance(fileids, compat.string_types):
+        if isinstance(fileids, string_types):
             fileids = [fileids]
         for f in fileids:
             if f not in self._fileids:
@@ -78,7 +79,7 @@ class YCOECorpusReader(CorpusReader):
         """
         if documents is None:
             return self._fileids
-        elif isinstance(documents, compat.string_types):
+        elif isinstance(documents, string_types):
             documents = [documents]
         return sorted(set(['%s.pos' % doc for doc in documents] +
                           ['%s.psd' % doc for doc in documents]))
@@ -91,7 +92,7 @@ class YCOECorpusReader(CorpusReader):
         if documents is None:
             documents = self._documents
         else:
-            if isinstance(documents, compat.string_types):
+            if isinstance(documents, string_types):
                 documents = [documents]
             for document in documents:
                 if document not in self._documents:
@@ -140,8 +141,8 @@ class YCOETaggedCorpusReader(TaggedCorpusReader):
 #: A list of all documents and their titles in ycoe.
 documents = {
     'coadrian.o34': 'Adrian and Ritheus',
-    'coaelhom.o3': 'lfric, Supplemental Homilies',
-    'coaelive.o3': 'lfric\'s Lives of Saints',
+    'coaelhom.o3': 'Ælfric, Supplemental Homilies',
+    'coaelive.o3': 'Ælfric\'s Lives of Saints',
     'coalcuin': 'Alcuin De virtutibus et vitiis',
     'coalex.o23': 'Alexander\'s Letter to Aristotle',
     'coapollo.o3': 'Apollonius of Tyre',
@@ -153,8 +154,8 @@ documents = {
     'cobyrhtf.o3': 'Byrhtferth\'s Manual',
     'cocanedgD': 'Canons of Edgar (D)',
     'cocanedgX': 'Canons of Edgar (X)',
-    'cocathom1.o3': 'lfric\'s Catholic Homilies I',
-    'cocathom2.o3': 'lfric\'s Catholic Homilies II',
+    'cocathom1.o3': 'Ælfric\'s Catholic Homilies I',
+    'cocathom2.o3': 'Ælfric\'s Catholic Homilies II',
     'cochad.o24': 'Saint Chad',
     'cochdrul': 'Chrodegang of Metz, Rule',
     'cochristoph': 'Saint Christopher',
@@ -173,7 +174,7 @@ documents = {
     'codocu4.o24': 'Documents 4 (O2/O4)',
     'coeluc1': 'Honorius of Autun, Elucidarium 1',
     'coeluc2': 'Honorius of Autun, Elucidarium 1',
-    'coepigen.o3': 'lfric\'s Epilogue to Genesis',
+    'coepigen.o3': 'Ælfric\'s Epilogue to Genesis',
     'coeuphr': 'Saint Euphrosyne',
     'coeust': 'Saint Eustace and his companions',
     'coexodusP': 'Exodus (P)',
@@ -188,8 +189,8 @@ documents = {
     'colaece.o2': 'Leechdoms',
     'colaw1cn.o3': 'Laws, Cnut I',
     'colaw2cn.o3': 'Laws, Cnut II',
-    'colaw5atr.o3': 'Laws, thelred V',
-    'colaw6atr.o3': 'Laws, thelred VI',
+    'colaw5atr.o3': 'Laws, Æthelred V',
+    'colaw6atr.o3': 'Laws, Æthelred VI',
     'colawaf.o2': 'Laws, Alfred',
     'colawafint.o2': 'Alfred\'s Introduction to Laws',
     'colawger.o34': 'Laws, Gerefa',
@@ -197,14 +198,14 @@ documents = {
     'colawnorthu.o3': 'Northumbra Preosta Lagu',
     'colawwllad.o4': 'Laws, William I, Lad',
     'coleofri.o4': 'Leofric',
-    'colsigef.o3': 'lfric\'s Letter to Sigefyrth',
-    'colsigewB': 'lfric\'s Letter to Sigeweard (B)',
-    'colsigewZ.o34': 'lfric\'s Letter to Sigeweard (Z)',
-    'colwgeat': 'lfric\'s Letter to Wulfgeat',
-    'colwsigeT': 'lfric\'s Letter to Wulfsige (T)',
-    'colwsigeXa.o34': 'lfric\'s Letter to Wulfsige (Xa)',
-    'colwstan1.o3': 'lfric\'s Letter to Wulfstan I',
-    'colwstan2.o3': 'lfric\'s Letter to Wulfstan II',
+    'colsigef.o3': 'Ælfric\'s Letter to Sigefyrth',
+    'colsigewB': 'Ælfric\'s Letter to Sigeweard (B)',
+    'colsigewZ.o34': 'Ælfric\'s Letter to Sigeweard (Z)',
+    'colwgeat': 'Ælfric\'s Letter to Wulfgeat',
+    'colwsigeT': 'Ælfric\'s Letter to Wulfsige (T)',
+    'colwsigeXa.o34': 'Ælfric\'s Letter to Wulfsige (Xa)',
+    'colwstan1.o3': 'Ælfric\'s Letter to Wulfstan I',
+    'colwstan2.o3': 'Ælfric\'s Letter to Wulfstan II',
     'comargaC.o34': 'Saint Margaret (C)',
     'comargaT': 'Saint Margaret (T)',
     'comart1': 'Martyrology, I',
@@ -219,11 +220,11 @@ documents = {
     'conicodE': 'Gospel of Nicodemus (E)',
     'coorosiu.o2': 'Orosius',
     'cootest.o3': 'Heptateuch',
-    'coprefcath1.o3': 'lfric\'s Preface to Catholic Homilies I',
-    'coprefcath2.o3': 'lfric\'s Preface to Catholic Homilies II',
+    'coprefcath1.o3': 'Ælfric\'s Preface to Catholic Homilies I',
+    'coprefcath2.o3': 'Ælfric\'s Preface to Catholic Homilies II',
     'coprefcura.o2': 'Preface to the Cura Pastoralis',
-    'coprefgen.o3': 'lfric\'s Preface to Genesis',
-    'copreflives.o3': 'lfric\'s Preface to Lives of Saints',
+    'coprefgen.o3': 'Ælfric\'s Preface to Genesis',
+    'copreflives.o3': 'Ælfric\'s Preface to Lives of Saints',
     'coprefsolilo': 'Preface to Augustine\'s Soliloquies',
     'coquadru.o23': 'Pseudo-Apuleius, Medicina de quadrupedibus',
     'corood': 'History of the Holy Rood-Tree',
@@ -231,7 +232,7 @@ documents = {
     'cosolilo': 'St. Augustine\'s Soliloquies',
     'cosolsat1.o4': 'Solomon and Saturn I',
     'cosolsat2': 'Solomon and Saturn II',
-    'cotempo.o3': 'lfric\'s De Temporibus Anni',
+    'cotempo.o3': 'Ælfric\'s De Temporibus Anni',
     'coverhom': 'Vercelli Homilies',
     'coverhomE': 'Vercelli Homilies (E)',
     'coverhomL': 'Vercelli Homilies (L)',
