@@ -1,11 +1,12 @@
 # Natural Language Toolkit: Translation metrics
 #
-# Copyright (C) 2001-2015 NLTK Project
+# Copyright (C) 2001-2017 NLTK Project
 # Author: Will Zhang <wilzzha@gmail.com>
 #         Guan Gui <ggui@student.unimelb.edu.au>
 #         Steven Bird <stevenbird1@gmail.com>
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
+from __future__ import division
 
 def alignment_error_rate(reference, hypothesis, possible=None):
     """
@@ -35,5 +36,5 @@ def alignment_error_rate(reference, hypothesis, possible=None):
     else:
         assert(reference.issubset(possible)) # sanity check
 
-    return (1.0 - float(len(hypothesis & reference) + len(hypothesis & possible)) /
+    return (1.0 - (len(hypothesis & reference) + len(hypothesis & possible)) /
             float(len(hypothesis) + len(reference)))
